// core_ex.h //

#ifndef __CORE_EX_H
#define __CORE_EX_H

#include <stdexcept>
#include <vector>

using std::runtime_error;
using std::string;
using std::vector;

/** @brief Bad position exception
 *
 * This exception indicates that a position outside the board was accessed
 */
class BadPosEx : public runtime_error
{
  public:
    /** constructor takes the position */
    BadPosEx(int iPos) : runtime_error ("Bad position")
    {
      m_iPos = iPos;
    }
    /// get the position
    int GetPos (void) { return m_iPos; }

  private:
    /// the bad position
    int m_iPos;
};

/** @brief Bad portal exception
 *
 * This exception indicates that an invalid portal was attempted to be created.
 * This exception may occur when a loop is detected.
 */
class BadPortalEx : public runtime_error
{
  public:
    /// constructor takes the start and end position
    BadPortalEx(int iSPos, int iEPos) : runtime_error ("Bad portal")
    {
      m_iSPos = iSPos;
      m_iEPos = iEPos;
    }
    /// get the start position
    int GetSPos (void) { return m_iSPos; }
    /// get the end position
    int GetEPos (void) { return m_iEPos; }

  private:
    /// the endpoints of the bad portal
    int m_iSPos, m_iEPos;
};

/** @brief a bad number of moves were attempted
 *
 * This can happen if the moves requested are outside range 1-6
 */
class BadMovesEx : public runtime_error
{
  public:
    /// constructor takes the number of moves requested
    BadMovesEx(int iMoves) : runtime_error ("Bad number of moves")
    {
      m_iMoves = iMoves;
    }
    /// get the number of moves
    int GetMoves (void) { return m_iMoves; }

  private:
    /// the number of moves requested
    int m_iMoves;
};

/** @brief over throw
 *
 * This can happen if the moves requested take the player off the board
 */
class OverThrowEx : public runtime_error
{
  public:
    /// constructor takes the number of moves requested
    OverThrowEx() : runtime_error ("Need exact moves to win")
    {
    }
};

/** @brief  no portal exception
 *
 * This exception indicates that there was no portal in the cell whose
 * end position was sought.
 */
class NoPortalEx : public runtime_error
{
  public:
    NoPortalEx () : runtime_error ("No portal at this cell") 
    {
    }
};

/** @brief bad board file error
 *
 * The file does not exist or is inaccessible
 */
class BadBoardFile : public runtime_error
{
  public:
    BadBoardFile(const string& rkoStrName) 
      : runtime_error ((string) "Could not open board file : " + rkoStrName)
    {
      m_oStrName = rkoStrName;
    }

    ~BadBoardFile() throw ()
    {
    }

  private:
    string m_oStrName;
};

/** @brief error building a board from a board file
 */
class BuildError : public runtime_error
{
  public:
    /// give a helpful message in the constructor
    BuildError (const string& rkoStrErr) : runtime_error (rkoStrErr)
    {
    }
};

/** @brief bad player name */
class BadPNameEx : public runtime_error
{
  public:
    /// constructor takes the player name
    BadPNameEx (const string& rkoStrName) 
      : runtime_error ("Invalid player name: '" + rkoStrName + "'")
    {
    }
};

/** @brief not enough players to start a game */
class NotEnoughPlayersEx : public runtime_error
{
  public:
    /// constructor
    NotEnoughPlayersEx (void) : runtime_error("At least 2 players are needed")
    {
    }
};

/** @brief a game was already on when the user tried to start it again */
class GameAlreadyOnEx : public runtime_error
{
  public:
    /// constructor
    GameAlreadyOnEx (void) : runtime_error ((string) "A game is already on," +
        "<ENTER> to roll die")
    {
    }
};

/** @brief a game was not on when the user tried to play it again */
class GameNotOnEx : public runtime_error
{
  public:
    /// constructor
    GameNotOnEx (void) : runtime_error ("You have to start a game first.")
    {
    }
};

/** allow to atart the game only if 6 is rolled on the die */
class GameStartEx : public runtime_error
{
  public:
    /// constructor
    GameStartEx (void) : runtime_error ("You need to roll 6 on the die to start.")
    {
    }
};

/** The game is over. Somebody won it */
class GameOverEx : public runtime_error
{
  public:
    /// constructor takes the winner's name
    GameOverEx (string oStrWinner, char cPID) 
      : runtime_error ((string) "Game over. " + "( " + cPID + " : " + 
          oStrWinner + " ) " + " won!!!" )
    {
    }
};

#endif //__CORE_EX_H

// core_ex.h //
